@@ -13,26 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import absolute_import
+from __future__ import print_function
+
 import argparse
-import json
 import sys
 
-import pkg_resources
-
+import pbr._compat.metadata
 import pbr.version
 
 
-def _get_metadata(package_name):
-    try:
-        return json.loads(
-            pkg_resources.get_distribution(
-                package_name).get_metadata('pbr.json'))
-    except pkg_resources.DistributionNotFound:
-        raise Exception('Package {0} not installed'.format(package_name))
-    except Exception:
-        return None
-
-
 def get_sha(args):
     sha = _get_info(args.name)['sha']
     if sha:
@@ -43,13 +33,17 @@ def get_info(args):
     if args.short:
         print("{version}".format(**_get_info(args.name)))
     else:
-        print("{name}\t{version}\t{released}\t{sha}".format(
-            **_get_info(args.name)))
+        print(
+            "{name}\t{version}\t{released}\t{sha}".format(
+                **_get_info(args.name)
+            )
+        )
 
 
-def _get_info(name):
-    metadata = _get_metadata(name)
-    version = pkg_resources.get_distribution(name).version
+def _get_info(package_name):
+    metadata = pbr._compat.metadata.get_metadata(package_name)
+    version = pbr._compat.metadata.get_version(package_name)
+
     if metadata:
         if metadata['is_release']:
             released = 'released'
@@ -67,13 +61,17 @@ def _get_info(name):
             for part in version_parts:
                 if not part.isdigit():
                     released = "pre-release"
-    return dict(name=name, version=version, sha=sha, released=released)
+
+    return {
+        'name': package_name,
+        'version': version,
+        'sha': sha,
+        'released': released,
+    }
 
 
 def freeze(args):
-    sorted_dists = sorted(pkg_resources.working_set,
-                          key=lambda dist: dist.project_name.lower())
-    for dist in sorted_dists:
+    for dist in pbr._compat.metadata.get_distributions():
         info = _get_info(dist.project_name)
         output = "{name}=={version}".format(**info)
         if info['sha']:
@@ -83,14 +81,21 @@ def freeze(args):
 
 def main():
     parser = argparse.ArgumentParser(
-        description='pbr: Python Build Reasonableness')
+        description='pbr: Python Build Reasonableness'
+    )
     parser.add_argument(
-        '-v', '--version', action='version',
-        version=str(pbr.version.VersionInfo('pbr')))
+        '-v',
+        '--version',
+        action='version',
+        version=str(pbr.version.VersionInfo('pbr')),
+    )
 
     subparsers = parser.add_subparsers(
-        title='commands', description='valid commands', help='additional help',
-        dest='cmd')
+        title='commands',
+        description='valid commands',
+        help='additional help',
+        dest='cmd',
+    )
     subparsers.required = True
 
     cmd_sha = subparsers.add_parser('sha', help='print sha of package')
@@ -98,14 +103,20 @@ def main():
     cmd_sha.add_argument('name', help='package to print sha of')
 
     cmd_info = subparsers.add_parser(
-        'info', help='print version info for package')
+        'info', help='print version info for package'
+    )
     cmd_info.set_defaults(func=get_info)
     cmd_info.add_argument('name', help='package to print info of')
-    cmd_info.add_argument('-s', '--short', action="store_true",
-                          help='only display package version')
+    cmd_info.add_argument(
+        '-s',
+        '--short',
+        action="store_true",
+        help='only display package version',
+    )
 
     cmd_freeze = subparsers.add_parser(
-        'freeze', help='print version info for all installed packages')
+        'freeze', help='print version info for all installed packages'
+    )
     cmd_freeze.set_defaults(func=freeze)
 
     args = parser.parse_args()
