@@ -43,14 +43,11 @@
 # USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 # DAMAGE.
 
-"""The code in this module is mostly copy/pasted out of the distutils2 source
-code, as recommended by Tarek Ziade.  As such, it may be subject to some change
-as distutils2 development continues, and will have to be kept up to date.
+# The code in this module is mostly copy/pasted out of the distutils2 source
+# code, as recommended by Tarek Ziade.
 
-I didn't want to use it directly from distutils2 itself, since I do not want it
-to be an installation dependency for our packages yet--it is still too unstable
-(the latest version on PyPI doesn't even install).
-"""
+from __future__ import absolute_import
+from __future__ import print_function
 
 # These first two imports are not used, but are needed to get around an
 # irritating Python bug that can crop up when using ./setup.py test.
@@ -68,19 +65,16 @@ import re
 import shlex
 import sys
 import traceback
+import warnings
 
-import distutils.ccompiler
 from distutils import errors
 from distutils import log
-import pkg_resources
+import setuptools
 from setuptools import dist as st_dist
 from setuptools import extension
 
-try:
-    import ConfigParser as configparser
-except ImportError:
-    import configparser
-
+from pbr._compat.five import ConfigParser
+import pbr._compat.packaging
 from pbr import extra_files
 import pbr.hooks
 
@@ -125,7 +119,7 @@ CFG_TO_PY_SETUP_ARGS = (
     (('files', 'namespace_packages'), 'namespace_packages'),
     (('files', 'data_files'), 'data_files'),
     (('files', 'scripts'), 'scripts'),
-    (('files', 'modules'), 'py_modules'),   # **
+    (('files', 'modules'), 'py_modules'),  # **
     (('global', 'commands'), 'cmdclass'),
     # Not supported in distutils2, but provided for
     # backwards compatibility with setuptools
@@ -135,24 +129,121 @@ CFG_TO_PY_SETUP_ARGS = (
     (('backwards_compat', 'include_package_data'), 'include_package_data'),
 )
 
+DEPRECATED_CFG = {
+    ('metadata', 'home_page'): (
+        "Use '[metadata] url' (setup.cfg) or '[project.urls]' "
+        "(pyproject.toml) instead"
+    ),
+    ('metadata', 'summary'): (
+        "Use '[metadata] description' (setup.cfg) or '[project] description' "
+        "(pyproject.toml) instead"
+    ),
+    ('metadata', 'description_file'): (
+        "Use '[metadata] long_description' (setup.cfg) or '[project] readme' "
+        "(pyproject.toml) instead"
+    ),
+    ('metadata', 'classifier'): (
+        "Use '[metadata] classifiers' (setup.cfg) or '[project] classifiers' "
+        "(pyproject.toml) instead"
+    ),
+    ('metadata', 'platform'): (
+        "Use '[metadata] platforms' (setup.cfg) or "
+        "'[tool.setuptools] platforms' (pyproject.toml) instead"
+    ),
+    ('metadata', 'requires_dist'): (
+        "Use '[options] install_requires' (setup.cfg) or "
+        "'[project] dependencies' (pyproject.toml) instead"
+    ),
+    ('metadata', 'setup_requires_dist'): (
+        "Use '[options] setup_requires' (setup.cfg) or "
+        "'[build-system] requires' (pyproject.toml) instead"
+    ),
+    ('metadata', 'python_requires'): (
+        "Use '[options] python_requires' (setup.cfg) or "
+        "'[project] requires-python' (pyproject.toml) instead"
+    ),
+    ('metadata', 'requires_python'): (
+        "Use '[options] python_requires' (setup.cfg) or "
+        "'[project] requires-python' (pyproject.toml) instead"
+    ),
+    ('metadata', 'provides_dist'): "This option is ignored by pip",
+    ('metadata', 'provides_extras'): "This option is ignored by pip",
+    ('metadata', 'obsoletes_dist'): "This option is ignored by pip",
+    ('files', 'packages_root'): (
+        "Use '[options] package_dir' (setup.cfg) or '[tools.setuptools] "
+        "package_dir' (pyproject.toml) instead"
+    ),
+    ('files', 'packages'): (
+        "Use '[options] packages' (setup.cfg) or '[tools.setuptools] "
+        "packages' (pyproject.toml) instead"
+    ),
+    ('files', 'package_data'): (
+        "Use '[options.package_data]' (setup.cfg) or "
+        "'[tool.setuptools.package-data]' (pyproject.toml) instead"
+    ),
+    ('files', 'namespace_packages'): (
+        "Use '[options] namespace_packages' (setup.cfg) or migrate to PEP "
+        "420-style namespace packages instead"
+    ),
+    ('files', 'data_files'): (
+        "For package data files, use '[options] package_data' (setup.cfg) "
+        "or '[tools.setuptools] package_data' (pyproject.toml) instead. "
+        "Support for non-package data files is deprecated in setuptools "
+        "and their use is discouraged. If necessary, use "
+        "'[options] data_files' (setup.cfg) or '[tools.setuptools] data-files'"
+        "(pyproject.toml) instead."
+    ),
+    ('files', 'scripts'): (
+        "Migrate to using the console_scripts entrypoint and use "
+        "'[options.entry_points]' (setup.cfg) or '[project.scripts]' "
+        "(pyproject.toml) instead"
+    ),
+    ('files', 'modules'): (
+        "Use '[options] py_modules' (setup.cfg) or '[tools.setuptools] "
+        "py-modules' (pyproject.toml) instead"
+    ),
+    ('backwards_compat', 'zip_safe'): (
+        "This option is obsolete as it was only relevant in the context of "
+        "eggs"
+    ),
+    ('backwards_compat', 'dependency_links'): (
+        "This option is ignored by pip starting from pip 19.0"
+    ),
+    ('backwards_compat', 'tests_require'): (
+        "This option is ignored by pip starting from pip 19.0"
+    ),
+    ('backwards_compat', 'include_package_data'): (
+        "Use '[options] include_package_data' (setup.cfg) or "
+        "'[tools.setuptools] include-package-data' (pyproject.toml) instead"
+    ),
+}
+
 # setup() arguments that can have multiple values in setup.cfg
-MULTI_FIELDS = ("classifiers",
-                "platforms",
-                "install_requires",
-                "provides",
-                "obsoletes",
-                "namespace_packages",
-                "packages",
-                "package_data",
-                "data_files",
-                "scripts",
-                "py_modules",
-                "dependency_links",
-                "setup_requires",
-                "tests_require",
-                "keywords",
-                "cmdclass",
-                "provides_extras")
+MULTI_FIELDS = (
+    "classifiers",
+    "platforms",
+    "install_requires",
+    "provides",
+    "obsoletes",
+    "namespace_packages",
+    "packages",
+    "package_data",
+    "data_files",
+    "scripts",
+    "py_modules",
+    "dependency_links",
+    "setup_requires",
+    "tests_require",
+    "keywords",
+    "cmdclass",
+    "provides_extras",
+)
+
+# a mapping of removed keywords to the version of setuptools that they were deprecated in
+REMOVED_KEYWORDS = {
+    # https://setuptools.pypa.io/en/stable/history.html#v72-0-0
+    'tests_requires': '72.0.0',
+}
 
 # setup() arguments that can have mapping values in setup.cfg
 MAP_FIELDS = ("project_urls",)
@@ -218,16 +309,13 @@ def cfg_to_args(path='setup.cfg', script_args=()):
     :raises DistutilsFileError:
         When the setup.cfg file is not found.
     """
-
     # The method source code really starts here.
-    if sys.version_info >= (3, 0):
-        parser = configparser.ConfigParser()
-    else:
-        parser = configparser.SafeConfigParser()
+    parser = ConfigParser()
 
     if not os.path.exists(path):
-        raise errors.DistutilsFileError("file '%s' does not exist" %
-                                        os.path.abspath(path))
+        raise errors.DistutilsFileError(
+            "file '%s' does not exist" % os.path.abspath(path)
+        )
     try:
         parser.read(path, encoding='utf-8')
     except TypeError:
@@ -235,7 +323,7 @@ def cfg_to_args(path='setup.cfg', script_args=()):
         parser.read(path)
     config = {}
     for section in parser.sections():
-        config[section] = dict()
+        config[section] = {}
         for k, value in parser.items(section):
             config[section][k.replace('-', '_')] = value
 
@@ -253,8 +341,10 @@ def cfg_to_args(path='setup.cfg', script_args=()):
     try:
         if setup_hooks:
             setup_hooks = [
-                hook for hook in split_multiline(setup_hooks)
-                if hook != 'pbr.hooks.setup_hook']
+                hook
+                for hook in split_multiline(setup_hooks)
+                if hook != 'pbr.hooks.setup_hook'
+            ]
             for hook in setup_hooks:
                 hook_fn = resolve_name(hook)
                 try:
@@ -263,8 +353,9 @@ def cfg_to_args(path='setup.cfg', script_args=()):
                     log.error('setup hook %s terminated the installation')
                 except Exception:
                     e = sys.exc_info()[1]
-                    log.error('setup hook %s raised exception: %s\n' %
-                              (hook, e))
+                    log.error(
+                        'setup hook %s raised exception: %s\n' % (hook, e)
+                    )
                     log.error(traceback.format_exc())
                     sys.exit(1)
 
@@ -277,7 +368,12 @@ def cfg_to_args(path='setup.cfg', script_args=()):
         kwargs['include_package_data'] = True
         kwargs['zip_safe'] = False
 
-        register_custom_compilers(config)
+        if has_get_option(config, 'global', 'compilers'):
+            warnings.warn(
+                'Support for custom compilers was removed in pbr 7.0 and the '
+                '\'[global] compilers\' option is now ignored.',
+                DeprecationWarning,
+            )
 
         ext_modules = get_extension_modules(config)
         if ext_modules:
@@ -300,6 +396,25 @@ def cfg_to_args(path='setup.cfg', script_args=()):
     return kwargs
 
 
+def _read_description_file(config):
+    """Handle the legacy 'description_file' option."""
+    description_files = has_get_option(config, 'metadata', 'description_file')
+    if not description_files:
+        return None
+
+    description_files = split_multiline(description_files)
+
+    data = ''
+    for filename in description_files:
+        description_file = io.open(filename, encoding='utf-8')
+        try:
+            data += description_file.read().strip() + '\n\n'
+        finally:
+            description_file.close()
+
+    return data
+
+
 def setup_cfg_to_setup_kwargs(config, script_args=()):
     """Convert config options to kwargs.
 
@@ -313,28 +428,54 @@ def setup_cfg_to_setup_kwargs(config, script_args=()):
     # parse env_markers.
     all_requirements = {}
 
+    # We want people to use description and long_description over summary and
+    # description but there is obvious overlap. If we see the both of the
+    # former being used, don't normalize
+    skip_description_normalization = False
+    if has_get_option(config, 'metadata', 'description') and (
+        has_get_option(config, 'metadata', 'long_description')
+        or has_get_option(config, 'metadata', 'description_file')
+    ):
+        kwargs['description'] = has_get_option(
+            config, 'metadata', 'description'
+        )
+        long_description = has_get_option(
+            config, 'metadata', 'long_description'
+        )
+        if long_description:
+            kwargs['long_description'] = long_description
+        else:
+            kwargs['long_description'] = _read_description_file(config)
+
+        skip_description_normalization = True
+
     for alias, arg in CFG_TO_PY_SETUP_ARGS:
         section, option = alias
 
+        if skip_description_normalization and alias in (
+            ('metadata', 'summary'),
+            ('metadata', 'description'),
+        ):
+            continue
+
         in_cfg_value = has_get_option(config, section, option)
-        if not in_cfg_value and arg == "long_description":
-            in_cfg_value = has_get_option(config, section, "description_file")
-            if in_cfg_value:
-                in_cfg_value = split_multiline(in_cfg_value)
-                value = ''
-                for filename in in_cfg_value:
-                    description_file = io.open(filename, encoding='utf-8')
-                    try:
-                        value += description_file.read().strip() + '\n\n'
-                    finally:
-                        description_file.close()
-                in_cfg_value = value
+
+        if alias == ('metadata', 'description') and not in_cfg_value:
+            in_cfg_value = _read_description_file(config)
 
         if not in_cfg_value:
             continue
 
+        if alias in DEPRECATED_CFG:
+            warnings.warn(
+                "The '[%s] %s' option is deprecated: %s"
+                % (alias[0], alias[1], DEPRECATED_CFG[alias]),
+                DeprecationWarning,
+            )
+
         if arg in CSV_FIELDS:
             in_cfg_value = split_csv(in_cfg_value)
+
         if arg in MULTI_FIELDS:
             in_cfg_value = split_multiline(in_cfg_value)
         elif arg in MAP_FIELDS:
@@ -351,17 +492,28 @@ def setup_cfg_to_setup_kwargs(config, script_args=()):
                 in_cfg_value = False
 
         if in_cfg_value:
+            if arg in REMOVED_KEYWORDS and (
+                pbr._compat.packaging.parse_version(setuptools.__version__)
+                >= pbr._compat.packaging.parse_version(REMOVED_KEYWORDS[arg])
+            ):
+                # deprecation warnings, if any, will already have been logged,
+                # so simply skip this
+                continue
+
             if arg in ('install_requires', 'tests_require'):
                 # Replaces PEP345-style version specs with the sort expected by
                 # setuptools
-                in_cfg_value = [_VERSION_SPEC_RE.sub(r'\1\2', pred)
-                                for pred in in_cfg_value]
+                in_cfg_value = [
+                    _VERSION_SPEC_RE.sub(r'\1\2', pred)
+                    for pred in in_cfg_value
+                ]
             if arg == 'install_requires':
                 # Split install_requires into package,env_marker tuples
                 # These will be re-assembled later
                 install_requires = []
                 requirement_pattern = (
-                    r'(?P<package>[^;]*);?(?P<env_marker>[^#]*?)(?:\s*#.*)?$')
+                    r'(?P<package>[^;]*);?(?P<env_marker>[^#]*?)(?:\s*#.*)?$'
+                )
                 for requirement in in_cfg_value:
                     m = re.match(requirement_pattern, requirement)
                     requirement_package = m.group('package').strip()
@@ -390,7 +542,8 @@ def setup_cfg_to_setup_kwargs(config, script_args=()):
                     elif firstline:
                         raise errors.DistutilsOptionError(
                             'malformed package_data first line %r (misses '
-                            '"=")' % line)
+                            '"=")' % line
+                        )
                     else:
                         prev.extend(shlex_split(line.strip()))
                     firstline = False
@@ -423,14 +576,17 @@ def setup_cfg_to_setup_kwargs(config, script_args=()):
 
     if 'extras' in config:
         requirement_pattern = (
-            r'(?P<package>[^:]*):?(?P<env_marker>[^#]*?)(?:\s*#.*)?$')
+            r'(?P<package>[^:]*):?(?P<env_marker>[^#]*?)(?:\s*#.*)?$'
+        )
         extras = config['extras']
         # Add contents of test-requirements, if any, into an extra named
         # 'test' if one does not already exist.
         if 'test' not in extras:
             from pbr import packaging
-            extras['test'] = "\n".join(packaging.parse_requirements(
-                packaging.TEST_REQUIREMENTS_FILES)).replace(';', ':')
+
+            extras['test'] = "\n".join(
+                packaging.parse_requirements(packaging.TEST_REQUIREMENTS_FILES)
+            ).replace(';', ':')
 
         for extra in extras:
             extra_requirements = []
@@ -461,7 +617,9 @@ def setup_cfg_to_setup_kwargs(config, script_args=()):
                 # multiple setup.py commands at once.
                 if 'bdist_wheel' not in script_args:
                     try:
-                        if pkg_resources.evaluate_marker('(%s)' % env_marker):
+                        if pbr._compat.packaging.evaluate_marker(
+                            '(%s)' % env_marker
+                        ):
                             extras_key = req_group
                     except SyntaxError:
                         log.error(
@@ -481,57 +639,24 @@ def setup_cfg_to_setup_kwargs(config, script_args=()):
     return kwargs
 
 
-def register_custom_compilers(config):
-    """Handle custom compilers.
-
-    This has no real equivalent in distutils, where additional compilers could
-    only be added programmatically, so we have to hack it in somehow.
-    """
-
-    compilers = has_get_option(config, 'global', 'compilers')
-    if compilers:
-        compilers = split_multiline(compilers)
-        for compiler in compilers:
-            compiler = resolve_name(compiler)
-
-            # In distutils2 compilers these class attributes exist; for
-            # distutils1 we just have to make something up
-            if hasattr(compiler, 'name'):
-                name = compiler.name
-            else:
-                name = compiler.__name__
-            if hasattr(compiler, 'description'):
-                desc = compiler.description
-            else:
-                desc = 'custom compiler %s' % name
-
-            module_name = compiler.__module__
-            # Note; this *will* override built in compilers with the same name
-            # TODO(embray): Maybe display a warning about this?
-            cc = distutils.ccompiler.compiler_class
-            cc[name] = (module_name, compiler.__name__, desc)
-
-            # HACK!!!!  Distutils assumes all compiler modules are in the
-            # distutils package
-            sys.modules['distutils.' + module_name] = sys.modules[module_name]
-
-
 def get_extension_modules(config):
     """Handle extension modules"""
 
-    EXTENSION_FIELDS = ("sources",
-                        "include_dirs",
-                        "define_macros",
-                        "undef_macros",
-                        "library_dirs",
-                        "libraries",
-                        "runtime_library_dirs",
-                        "extra_objects",
-                        "extra_compile_args",
-                        "extra_link_args",
-                        "export_symbols",
-                        "swig_opts",
-                        "depends")
+    EXTENSION_FIELDS = (
+        "sources",
+        "include_dirs",
+        "define_macros",
+        "undef_macros",
+        "library_dirs",
+        "libraries",
+        "runtime_library_dirs",
+        "extra_objects",
+        "extra_compile_args",
+        "extra_link_args",
+        "export_symbols",
+        "swig_opts",
+        "depends",
+    )
 
     ext_modules = []
     for section in config:
@@ -564,24 +689,30 @@ def get_extension_modules(config):
             if ext_args:
                 if 'name' not in ext_args:
                     ext_args['name'] = labels[1]
-                ext_modules.append(extension.Extension(ext_args.pop('name'),
-                                                       **ext_args))
+                ext_modules.append(
+                    extension.Extension(ext_args.pop('name'), **ext_args)
+                )
     return ext_modules
 
 
 def get_entry_points(config):
-    """Process the [entry_points] section of setup.cfg.
-
-    Processes setup.cfg to handle setuptools entry points. This is, of course,
-    not a standard feature of distutils2/packaging, but as there is not
-    currently a standard alternative in packaging, we provide support for them.
-    """
+    """Process the [entry_points] section of setup.cfg."""
 
     if 'entry_points' not in config:
         return {}
 
-    return dict((option, split_multiline(value))
-                for option, value in config['entry_points'].items())
+    warnings.warn(
+        "The 'entry_points' section has been deprecated in favour of the "
+        "'[options.entry_points]' section (if using 'setup.cfg') or the "
+        "'[project.scripts]' and/or '[project.entry-points.{name}]' sections "
+        "(if using 'pyproject.toml')",
+        DeprecationWarning,
+    )
+
+    return {
+        option: split_multiline(value)
+        for option, value in config['entry_points'].items()
+    }
 
 
 def has_get_option(config, section, option):
@@ -594,18 +725,22 @@ def has_get_option(config, section, option):
 def split_multiline(value):
     """Special behaviour when we have a multi line options"""
 
-    value = [element for element in
-             (line.strip() for line in value.split('\n'))
-             if element and not element.startswith('#')]
+    value = [
+        element
+        for element in (line.strip() for line in value.split('\n'))
+        if element and not element.startswith('#')
+    ]
     return value
 
 
 def split_csv(value):
     """Special behaviour when we have a comma separated options"""
 
-    value = [element for element in
-             (chunk.strip() for chunk in value.split(','))
-             if element]
+    value = [
+        element
+        for element in (chunk.strip() for chunk in value.split(','))
+        if element
+    ]
     return value
 
 
